#include "Assembler.hpp"
#include "Instruction.hpp"

#include "InstructionSet.hpp"
#include "Token.hpp"
#include "Error.hpp"

#include <stdio.h>
#include <string.h>
#include <io.h>

namespace SoftWire
{
	Assembler::Assembler(const char *sourceFile)
	{
		assembleFile(sourceFile);
	}

	Assembler::~Assembler()
	{
	}

	void Assembler::assembleFile(const char *sourceFile)
	{
		FILE *file = fopen(sourceFile, "rb");

		if(!file)
		{
			printf("Could not open source file: '%s'\n", sourceFile);
			return;
		}
		else
		{
			printf("Assembling '%s'...\n\n", sourceFile);
		}

		int length = _filelength(_fileno(file));

		char *buffer = new char[length + 1];
		fread(buffer, sizeof(char), length, file);
		fclose(file);
		buffer[length] = 0;

		char *sourceLine = buffer;

		lineNumber = 1;
		error = false;

		while(*sourceLine)
		{
			assembleLine(sourceLine);

			while(*sourceLine != '\0' && *sourceLine != '\n' && *sourceLine != '\r') sourceLine++;
			while(*sourceLine == '\n' || *sourceLine == '\r') sourceLine++;
		}

		delete[] buffer;
	}

	const void *Assembler::callable()
	{
		if(error)
		{
			return 0;
		}
		else
		{
			return loader.callable();
		}
	}

	void Assembler::assembleLine(const char *sourceLine)
	{
		try
		{
			const Encoding &encoding = parser.parseLine(sourceLine);

			loader.addEncoding(encoding);
		}
		catch(Error &errorMessage)
		{
			error = true;

			errorMessage.print();
			
			printLine(sourceLine);
		}
		catch(...)
		{
			printf("error: Unhandled exception\n");
		}
	}

	void Assembler::printLine(const char *string)
	{
		while(*string != '\0' && *string != '\n')
		{
			putchar(*string++);
		}
		putchar('\n');
	}
};