#ifndef Instruction_hpp
#define Instruction_hpp

#include "Keywords.hpp"

namespace SoftWire
{
	class Instruction
	{
	public:
		enum
		{
			CPU_UNKNOWN	= 0x00000000,

			CPU_8086	= 0x00000001,
			CPU_186		= 0x00000002,
			CPU_286		= 0x00000004,
			CPU_386		= 0x00000008,
			CPU_486		= 0x00000010,
			CPU_PENT	= 0x00000020,   // Pentium
			CPU_P6		= 0x00000040,   // Pentium Pro

			CPU_FPU		= 0x00000080,
			CPU_MMX		= 0x00000100,
			CPU_KATMAI	= 0x00000200,
			CPU_SSE		= 0x00000400,

		//	CPU_AMD		= 0x00000800,   // AMD specific system calls
			CPU_CYRIX	= 0x00001000,
			CPU_3DNOW	= 0x00002000,
			CPU_ATHLON	= 0x00004000,
		//	CPU_SMM		= 0x00008000,   // System Management Mode, standby mode

		//	CPU_UNDOC	= 0x00010000,   // Undocumented, also not supported by inline assembler
		//	CPU_PRIV	= 0x00020000    // Priviledged, run-time compiled OS kernel anyone?
		};

		struct Syntax
		{
			char *mnemonic;
			char *operands;
			char *encoding;
			int flags;
		};

		Instruction(const Syntax &instruction);

		~Instruction();

		Instruction *getNext() const;

		void attachNew(const Syntax &instruction);
		
		void resetMatch();
		bool matchSyntax() const;
		void matchMnemonic(const char *mnemonic);
		void matchSpecifier(Specifier::Type sizeSpecifier);
		void matchFirstOperand(const Operand &operand);
		void matchSecondOperand(const Operand &operand);
		void matchThirdOperand(const Operand &operand);

		Operand::Type getFirstOperand() const;
		Operand::Type getSecondOperand() const;
		Operand::Type getThirdOperand() const;

		const char *getMnemonic() const;
		const char *getOperandSyntax() const;
		const char *getEncoding() const;
		
		bool is32Bit() const;

	private:
		bool syntaxMnemonic : 1;
		bool syntaxSpecifier : 1;
		bool syntaxFirstOperand : 1;
		bool syntaxSecondOperand : 1;
		bool syntaxThirdOperand : 1;

		const Syntax &syntax;
		Specifier::Type specifier;
		Operand::Type firstOperand;
		Operand::Type secondOperand;
		Operand::Type thirdOperand;
		int flags;

		Instruction *next;

		void extractOperands(const char *syntax);
	};
}

#endif   // Instruction_hpp